/*!
 * Copyright (C) 2003 by Frank Wallenwein. All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY FRANK WALLENWEIN
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*
 * ***** EA DIP204-4 LCD Display on Ethernut ********
 */

/*!
 * \example lcd/lcd.c
 *
 * This sample demonstrates the usage of KS0073 driver 
 * with EA DIP204-4 LCD Dispaly ( 4 Lines, 20 Characters/line )
 *
 * The LCD Display is connected via serial lines. 
 * I used the following lines : 
 * 
 * LCD            Ethernut
 *
 * Pin 1 VSS   --> Gnd   
 * Pin 2 VDD   --> VCC   
 * Pin 5 SID   --> PD0   
 * Pin 6 SCLK  --> PD2
 * Pin 7 SOD   --> PD4  ( not used ) 
 *
 * Pin 4 (Chip Select) is connected to Pin 1 (GND)
 * 
 * So we realy need just TWO I/O Ports from Ethernut 
 * to controll a Display. 
 * One more Port could be used to switch Display ilumination on/off
 * One more Port could be used to read back from Display.
 * 
 * Information on the Display can be found at
 * http://www.lcd-module.de ( Click at Dotmatix ) 
 * http://www.conrad.de ( Part No. 181863-14 )
 *
 */

#include <string.h>
#include <stdio.h>
#include <io.h>

#include <dev/uartavr.h>
#include <dev/ks0073.h>
#include <sys/timer.h>

static char *banner          = "Nut/OS LCD\n\r";
static prog_char head1[]     = "\r!******************!\r\n";
static prog_char head2[]     = "\r! Nut/OS serial LCD!\r\n";
static prog_char head3[]     = "\r! Your Name is :   !\r\n";
static prog_char head4[]     = "\r  Name: UNKNOWN\r";

static prog_char presskey_P[]= "Press any key...";
static prog_char pgm_ptr[]   = "\nHello stranger!\n";

static char inbuf[128];

/*
 * LCD sample.
 *
 * Some functions do not work with ICCAVR.
 */
int main(void)
{
    int got;
    int i;
    char *cp;
    u_long baud = 38400;
    FILE *uart;
    FILE *lcd;
    float dval = 0.0;

    /*
     * Each device must be registered. We do this by referencing the 
     * device structure of the driver. The advantage is, that only 
     * those device drivers are included in our flash code, which we 
     * really need.
     *
     * The uart0 device is the first one on the ATmega chip. So it 
     * has no configurable base address or interrupt and we set both 
     * parameters to zero.
     */
    NutRegisterDevice(&devUart0, 0, 0);

    /* 
     * Register serial LCD Display driver 
     */
     
    NutRegisterDevice(&devLcdSer, 0, 0);

    /*
     * Now, as the devices are registered, we can open them. The fopen()
     * function returns a pointer to a FILE structure, which we use 
     * for subsequent reading and writing.
     */
    uart = fopen("uart0", "r+");
    lcd  = fopen("lcdser", "r+");

    /*
     * Before doing the first read or write, we set the baudrate.
     * This low level function doesn't know about FILE structures
     * and we use _fileno() to get the low level file descriptor
     * of the stream.
     *
     * The short sleep allows the UART to settle after the baudrate
     * change.
     */
     _ioctl(_fileno(uart), UART_SETSPEED, &baud);

    /*
     * Stream devices can use low level read and write functions. 
     * Writing program space data is supported too.
     */
    _write(_fileno(uart), banner, strlen(banner));
    {
        _write_P(_fileno(uart), presskey_P, sizeof(presskey_P));
    }

    /*
     * Stream devices do buffered I/O. That means, nothing will be 
     * passed to the hardware device until either the output buffer 
     * is full or we do a flush. With stream I/O we typically use
     * fflush(), but low level writing a null pointer will also flush 
     * the output buffer.
     */
    _write(_fileno(uart), 0, 0);

    _write_P(_fileno(lcd), head1, sizeof(head1));
    _write_P(_fileno(lcd), head2, sizeof(head2));
    _write_P(_fileno(lcd), head1, sizeof(head1));        
    _write_P(_fileno(lcd), head4, sizeof(head4));
    _write(_fileno(lcd), 0, 0);

    /*
     * The low level function read() will grab all available bytes 
     * from the input buffer. If the buffer is empty, the call will
     * block until something is available for reading.
     */
    got = _read(_fileno(uart), inbuf, sizeof(inbuf));
    _write(_fileno(uart), inbuf, got);

    /*
     * Nut/OS never expects a thread to return. So we enter an 
     * endless loop here.
     */
    for (i = 0;; i++) {
        /*
         * A bit more advanced input routine is able to read a string 
         * up to and including the first newline character or until a
         * specified maximum number of characters, whichever comes first.
         */
        fputs("\nEnter your name: ", uart);
        fflush(uart);
        fgets(inbuf, sizeof(inbuf), uart);

        /*
         * Chop off trailing linefeed.
         */
        cp = strchr(inbuf, '\n');
        if (cp)
            *cp = 0;

        /*
         * Streams support formatted output as well as printing strings 
         * from program space.
         */
        if (inbuf[0])
            fprintf(uart, "\nHello %s!\n", inbuf);
        else {
            fputs_P(pgm_ptr, uart);
        }
        if (inbuf[0])
        {
            fprintf(lcd, "\r%s", inbuf);
            fprintf(lcd,"%cK",27);   /* ESC-K Erase to end of line */
            _write(_fileno(lcd), 0, 0);
        }
        else {
            fputs_P(head4, lcd);
        }
        

        /*
         * Just to demonstrate formatted floating point output.
         * In order to use this, we need to link the application
         * with nutcrtf instead of nutcrt for pure integer.
         */
        dval += 1.0125;
        fprintf(uart, "FP %f\n",(double) dval);
    }
}
